"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs-p");
const _ = require("lodash");
const typescript = require("./typescript");
// Folders
const serverlessFolder = '.serverless';
const buildFolder = '.build';
class ServerlessPlugin {
    constructor(serverless, options) {
        this.serverless = serverless;
        this.options = options;
        this.hooks = {
            'before:deploy:createDeploymentArtifacts': this.beforeCreateDeploymentArtifacts.bind(this),
            'after:deploy:createDeploymentArtifacts': this.afterCreateDeploymentArtifacts.bind(this),
        };
    }
    beforeCreateDeploymentArtifacts() {
        return __awaiter(this, void 0, void 0, function* () {
            this.serverless.cli.log('Compiling with Typescript...');
            // Save original service path and functions
            this.originalServicePath = this.serverless.config.servicePath;
            // Fake service path so that serverless will know what to zip
            this.serverless.config.servicePath = path.join(this.originalServicePath, buildFolder);
            const tsFileNames = typescript.extractFileNames(this.serverless.service.functions);
            const tsconfig = typescript.getTypescriptConfig(this.originalServicePath);
            for (const fnName in this.serverless.service.functions) {
                const fn = this.serverless.service.functions[fnName];
                fn.package = fn.package || {
                    exclude: [],
                    include: [],
                };
                fn.package.exclude = _.uniq([...fn.package.exclude, 'node_modules/serverless-plugin-typescript']);
            }
            tsconfig.outDir = buildFolder;
            yield typescript.run(tsFileNames, tsconfig);
            // include node_modules into build
            fs.symlinkSync(path.resolve('node_modules'), path.resolve(path.join(buildFolder, 'node_modules')));
        });
    }
    afterCreateDeploymentArtifacts() {
        return __awaiter(this, void 0, void 0, function* () {
            // Restore service path
            this.serverless.config.servicePath = this.originalServicePath;
            // Copy .build to .serverless
            yield fs.copy(path.join(this.originalServicePath, buildFolder, serverlessFolder), path.join(this.originalServicePath, serverlessFolder));
            this.serverless.service.package.artifact = path.join(this.originalServicePath, serverlessFolder, path.basename(this.serverless.service.package.artifact));
            // Remove temp build folder
            fs.removeSync(path.join(this.originalServicePath, buildFolder));
        });
    }
}
module.exports = ServerlessPlugin;
//# sourceMappingURL=index.js.map